package jalview.schemes;

import jalview.datamodel.*;
import java.util.*;
import java.awt.*;

public class ResidueColourScheme implements ColourSchemeI {
    Color    [] colors;
    int         threshold = 90;

    public ResidueColourScheme(Color[] colors, int threshold) {
	this.colors    = colors;
	this.threshold = threshold;
    }

    public ResidueColourScheme() {
    }

    public Color findColour(String aa)
    {
      return colors[((Integer)(ResidueProperties.aaHash.get(aa))).intValue()];
    }

    public Color findColour(SequenceI seq,String s, int j, Vector aa) {
  try {
	    return colors[((Integer)(ResidueProperties.aaHash.get(s))).intValue()];
	} catch (Exception e) {
	    return Color.white;
	}
    }

    // aa should maybe be a class
    public Color getColour(SequenceI seq, int j,Vector aa) {

	Color  c       = Color.white;
	String s       = seq.getSequence(j,j+1);

	if (threshold > 0 && aa != null)
        {
	    if (aboveThreshold(aa,seq,j,threshold))
		c = findColour(seq,s,j,aa);
	}
        else
	    c = findColour(seq,s,j,aa);


      return c;
    }
    public int getThreshold() {
	return threshold;
    }

    public void setThreshold(int ct) {
	threshold = ct;
    }

    public Vector  getColours(SequenceI s, Vector aa) {
	Vector colours = new Vector();

	for (int j = 0; j < s.getLength(); j++)
	    colours.addElement(getColour(s,j,aa));

	return colours;
    }

    public Vector getColours(SequenceGroup sg, Vector aa) {
	Vector colours = new Vector();

	for (int j = 0; j < sg.getSize(); j++) {
	    SequenceI s = sg.getSequenceAt(j);

	    for (int i = 0; i < s.getLength();i++) {
		colours.addElement(getColour(s,i,aa));
	    }
	}
	return colours;
    }

    public boolean aboveThreshold(Vector aa,SequenceI seq, int j, int threshold) {
	String    s    = seq.getSequence(j,j+1);
	Hashtable hash = (Hashtable)aa.elementAt(j);

	if (j < aa.size()) {
	    String maxRes = (String)hash.get("maxResidue");

	    double sc = 0;

	    if (((Integer)hash.get("maxCount")).intValue() != -1  && hash.contains(s)) {
		int maxCount = ((Integer)hash.get("maxCount")).intValue();
		int resCount = ((Integer)hash.get(s)).intValue();

		sc = resCount * 100.0 / resCount;

		// This should be isGap somewhere
		if  ( !s.equals("-")  && !s.equals(".") && !s.equals(" ")) {
		    if (sc >= (double)threshold) {
			return true;
		    }
		}
	    }
	}
	return false;
    }

    public boolean canThreshold() {
	return true;
    }
    public boolean isUserDefinable() {
	return false;
    }
}
